#include "Zoomer.h"

#include <windows.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include <assert.h>

const double Zoomer::THROW_AWAY_ZOOM_TOLERANCE = 0.05;

Zoomer::Zoomer(void)
		:m_zoom_completed(true)
{
	static const double DEFAULT_LEFT	= +0.0;
	static const double DEFAULT_RIGHT	= +20.0;

	static const double DEFAULT_BOTTOM	= -100.0;
	static const double DEFAULT_TOP		= +100.0;

	m_zoom_history.push(ZoomDimensions(DEFAULT_LEFT, DEFAULT_RIGHT, DEFAULT_BOTTOM, DEFAULT_TOP));
}

void Zoomer::setupObjectSpaceDimensions(const double & left, const double & right, const double & bottom, const double & top)
{
	while (! m_zoom_history.empty())
	{
		m_zoom_history.pop();
	}

	m_zoom_history.push(ZoomDimensions(left, right, bottom, top));
}

void Zoomer::startZoomSelection(const double & normalized_left, const double & normalized_bottom)
{
	m_incomplete_zoom.setLeftBottom(getObjectSpaceX(normalized_left), getObjectSpaceY(normalized_bottom));

	m_zoom_completed = false;
}

void Zoomer::finishZoomSelection(const double & normalized_right, const double & normalized_top)
{
	m_incomplete_zoom.setRightTop(getObjectSpaceX(normalized_right), getObjectSpaceY(normalized_top));

	m_zoom_completed = false;
}

void Zoomer::pushZoom(void)
{
	m_zoom_completed = true;

	m_incomplete_zoom.swapIfOutOfOrder();

	double right	= m_incomplete_zoom.getRight();
	double left		= m_incomplete_zoom.getLeft();

	double width	= right - left;

	double top		= m_incomplete_zoom.getTop();
	double bottom	= m_incomplete_zoom.getBottom();

	double height = top - bottom;

	bool too_small_horizontal = ((width / getHorizontalRange()) < THROW_AWAY_ZOOM_TOLERANCE);
	bool too_small_vertical   = ((height / getVerticalRange()) < THROW_AWAY_ZOOM_TOLERANCE);

	if (! (too_small_horizontal || too_small_vertical))
	{
		m_zoom_history.push(m_incomplete_zoom);
	}
}

void Zoomer::popZoom(void)
{
	m_zoom_history.pop();
}

void Zoomer::applyTopZoom(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();
	
	glMatrixMode(GL_PROJECTION);

	glLoadIdentity();

	gluOrtho2D(current_zoom.getLeft(), current_zoom.getRight(), current_zoom.getBottom(), current_zoom.getTop());

	glMatrixMode(GL_MODELVIEW);
}

double Zoomer::getObjectSpaceX(const double & normalized_x) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	double right = current_zoom.getRight();

	double left = current_zoom.getLeft();

	double object_space_x = normalized_x * (right - left) + left;

	assert(object_space_x >= left);

	assert(object_space_x <= right);

	return object_space_x;
}

double Zoomer::getObjectSpaceY(const double & normalized_y) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	double top = current_zoom.getTop();

	double bottom = current_zoom.getBottom();

	double object_space_y = normalized_y * (top - bottom) + bottom;

	assert(object_space_y >= bottom);

	assert(object_space_y <= top);

	return object_space_y;
}

bool Zoomer::canZoomOut(void) const
{
	return (m_zoom_history.size() > 1);
}

double Zoomer::getLeft(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return current_zoom.getLeft();
}

double Zoomer::getRight(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return current_zoom.getRight();
}

double Zoomer::getBottom(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return current_zoom.getBottom();
}

double Zoomer::getTop(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return current_zoom.getTop();
}

double Zoomer::getHorizontalRange(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return (current_zoom.getRight() - current_zoom.getLeft());
}

double Zoomer::getVerticalRange(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return (current_zoom.getTop() - current_zoom.getBottom());
}

bool Zoomer::isZoomCompleted(void) const
{
	return m_zoom_completed;
}

double Zoomer::getIncompleteZoomLeft(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return m_incomplete_zoom.getLeft();
}

double Zoomer::getIncompleteZoomRight(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return m_incomplete_zoom.getRight();
}

double Zoomer::getIncompleteZoomBottom(void) const
{
	ZoomDimensions current_zoom = m_zoom_history.top();

	return m_incomplete_zoom.getBottom();
}

double Zoomer::getIncompleteZoomTop(void) const
{
	return m_incomplete_zoom.getTop();
}

